using DSPopt
using Test
using SparseArrays

const dsp = DSPopt.dspenv

@testset "Initializing DSPopt" begin
    @test dsp.p != C_NULL
    @test length(dsp.numRows) == 0
    @test length(dsp.numCols) == 0
    @test isnan(dsp.primVal)
    @test isnan(dsp.dualVal)
    @test length(dsp.colVal) == 0
    @test length(dsp.rowVal) == 0
    @test dsp.nblocks == -1
    @test dsp.block_ids == []
    @test dsp.is_stochastic == false
    @test dsp.solve_type == DSPopt.DW
    @test isnothing(dsp.comm)
    @test dsp.comm_size == 1
    @test dsp.comm_rank == 0
end

@testset "Check DSP versions" begin
    major = DSPopt.getVersionMajor(dsp)
    minor = DSPopt.getVersionMinor(dsp)
    patch = DSPopt.getVersionPatch(dsp)
    if major > 1
        @test true
    elseif major == 1
        if minor > 4 || (minor == 4 && patch >= 1)
            @test true
        else
            @test false
        end
    else
        @test false
    end
    @test DSPopt.getVersion(dsp) == "$(major).$(minor).$(patch)"
end

@testset "Setting options" begin
    @testset "param:" begin
        DSPopt.setoptions!(Dict(:param => "params.txt"))
    end
    @testset "is_stochastic: $i" for i in [true, false]
        DSPopt.setoptions!(Dict(:is_stochastic => i))
        @test dsp.is_stochastic == i
    end
    @testset "solve_type: $t" for t in instances(DSPopt.Methods)
        DSPopt.setoptions!(Dict(:solve_type => t))
        @test dsp.solve_type == t
    end
end

@testset "Farmer example: stochastic form" begin
    include("farmer_stoc.jl")
end

@testset "Farmer example: block form" begin
    include("farmer_block.jl")
end

@testset "dcap" begin
    include("dcap.jl")
end

@testset "Distributionally robust extension" begin
    include("Dro.jl")
end

@testset "Freeing DSPopt" begin
    DSPopt.freeEnv(dsp)
    @test dsp.p == C_NULL
    @test length(dsp.numRows) == 0
    @test length(dsp.numCols) == 0
    @test isnan(dsp.primVal)
    @test isnan(dsp.dualVal)
    @test length(dsp.colVal) == 0
    @test length(dsp.rowVal) == 0
    @test dsp.nblocks == -1
    @test dsp.block_ids == []
    @test dsp.is_stochastic == false
    @test dsp.solve_type == DSPopt.DW
    @test isnothing(dsp.comm)
    @test dsp.comm_size == 1
    @test dsp.comm_rank == 0
end
